#pragma once

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the IPIMOCAPAUTOMATION_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// IPIMOCAPAUTOMATION_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef IPIMOCAPAUTOMATION_EXPORTS
#define IPIMOCAPAUTOMATION_API __declspec(dllexport)
#else
#define IPIMOCAPAUTOMATION_API  __declspec(dllimport)
#endif

#include "iPiTypes.h"

// Connects to iPi Recorder. Starts the application if it is not already running and suppressAutoStart is set to false.
// PARAMETERS
//     executablePath: Path to an executable file for the application (iPiRecorder.exe). 
//     fnEventCallback: Pointer to a function that is called to handle incoming events on the connection.
//     phConnection: Pointer to a variable that receives a connection handle.
// RETURN VALUES
// On success returns IPI_OK.
// REMARKS
// When executablePath is NULL, then executable is searched via registered file association for iPiVideo files and in default installation folder.
// Events are messages initiated by an application on its own, not as direct answer to a previous command request from a client. They usually signal the end of some asynchronous activity. Event callback is called on a background thread.
// On failure variable at phConnection is set to INVALID_HANDLE_VALUE.
// When done with connection, call iPi_Disconnect.
IPIMOCAPAUTOMATION_API  iPi_Result iPi_ConnectToRecorder(LPCWSTR executablePath, iPi_EventCallback fnEventCallback, LPHANDLE phConnection, bool suppressAutoStart = false);

// Connects to iPi Mocap Studio. Starts the application if it is not already running and suppressAutoStart is set to false.
// PARAMETERS
//     executablePath: Path to an executable file for the application (iPiMocapStudio.exe). 
//     fnEventCallback: Pointer to a function that is called to handle incoming events on the connection.
//     phConnection: Pointer to a variable that receives a connection handle.
// RETURN VALUES
// On success returns IPI_OK. 
// REMARKS
// When executablePath is NULL, then executable is searched via registered file association for iPiMotion files and in default installation folder.
// Events are messages initiated by an application on its own, not as direct answer to a previous command request from a client. They usually signal the end of some asynchronous activity. Event callback is called on a background thread.
// On failure variable at phConnection is set to INVALID_HANDLE_VALUE.
// When done with connection, call iPi_Disconnect.
IPIMOCAPAUTOMATION_API  iPi_Result iPi_ConnectToMocapStudio(LPCWSTR executablePath, iPi_EventCallback fnEventCallback, LPHANDLE phConnection, bool suppressAutoStart = false);

// Closes a connection to iPi Recorder or iPi Mocap Studio.
// PARAMETERS
//     hConnection: Connection handle.
// RETURN VALUES
// On success returns IPI_OK. 
IPIMOCAPAUTOMATION_API  iPi_Result iPi_Disconnect(HANDLE hConnection);

// Executes a command via connection to an application.
// PARAMETERS
//     hConnection: Connection handle.
//     jsonCommand: String with JSON representation of the command.
//     pJsonResponse: Pointer to variable to receive a string with JSON representation of response to the command.
// RETURN VALUES
// On success returns IPI_OK.
// REMARKS
// Structure of command request is:
//     {
//         "command": "command_name",
//         "arguments": { /* optional */
//                          "arg_name1": arg_value1,
//                          ...
//                          "arg_nameN": arg_valueN
//                      }
//     }
// Structure of response is:
//     {
//         "success": true/false,
//         "error": "error_message", /* optional */
//         "result": scalar or structure /* optional */
//     }
// Pointer written to pJsonResponse should be freed with iPi_Free function.
IPIMOCAPAUTOMATION_API  iPi_Result iPi_ExecuteCommand(HANDLE hConnection, LPCWSTR jsonCommand, LPWSTR* pJsonResponse);

// Should be used by a caller to free memory allocated by this library.
// PARAMETERS
//     ptr: Pointer to a memory block to be freed, or NULL.
// RETURN VALUE
// TRUE if memory has been freed successfully, or ptr is NULL. Otherwise FALSE.
IPIMOCAPAUTOMATION_API  BOOL iPi_Free(LPVOID ptr);

// Returns full path to iPiRecorder executable.
// RETURN VALUES
// Returns full path if executable found or NULL otherwise.
// REMARKS
// Executable is searched via registered file association for iPiVideo files and in default installation folder.
// This function can be used if you start iPiRecorder from other app.
IPIMOCAPAUTOMATION_API  LPWSTR iPi_GetRecorderProgramPath();

// Returns full path to iPiMocapStudio executable.
// RETURN VALUES
// Returns full path if executable found or NULL otherwise.
// REMARKS
// Executable is searched via registered file association for iPiMotion files and in default installation folder.
// This function can be used if you start iPiMocapStudio from other app.
IPIMOCAPAUTOMATION_API  LPWSTR iPi_GetMocapStudioProgramPath();
